@@ -411,6 +411,7 @@ typedef struct {
 	/* For writing */
 	gchar *filename;
 	gint folder;
+	gint space_left;
 } FileHandle;
 
 static GnomeVFSResult
@@ -535,6 +536,8 @@ do_open (GnomeVFSMethod *method,
 		handle->handle = local_handle;
 		handle->folder = folder;
 		handle->filename = gnome_vfs_get_local_path_from_uri (local);
+		handle->space_left = (card == 0 ? rio->int_mem_free :
+				rio->ext_mem_free);
 
 		g_free (local);
 	}
@@ -564,7 +567,7 @@ do_close (GnomeVFSMethod *method,
 			res = GNOME_VFS_ERROR_IO;
 		else
 			res = GNOME_VFS_OK;
-	} else {
+	} else if (handle->space_left > 0) {
 		gint tmp;
 
 		DEBUG_RIO(("do_close(): uploading song %s to folder %d\n",
@@ -581,6 +584,10 @@ do_close (GnomeVFSMethod *method,
 
 		unlink (handle->filename);
 		res = rio_to_vfs_error(tmp);
+	} else {
+		DEBUG_RIO(("do_close(): not uploading, no memory\n"));
+		unlink (handle->filename);
+		res = GNOME_VFS_ERROR_NO_SPACE;
 	}
 
 	/* delete the handle's content */
@@ -618,12 +625,28 @@ do_write (GnomeVFSMethod *method,
 
 
 {
+	GnomeVFSResult result;
 	FileHandle *handle = (FileHandle *)method_handle;
 
 	DEBUG_RIO (("do_write() %p\n", method_handle));
 
-	return gnome_vfs_write (handle->handle, buffer, num_bytes,
+	/* Check if we have enough space left for that much on the rio */
+	DEBUG_RIO (("do_write() space left: %d / bytes: %d / diff: %d\n",
+				handle->space_left, num_bytes,
+				handle->space_left - num_bytes));
+
+	if ((handle->space_left - num_bytes) <= 0)
+	{
+		/* Set it so that it's negative */
+		handle->space_left -= num_bytes;
+		return GNOME_VFS_ERROR_NO_SPACE;
+	}
+
+	result = gnome_vfs_write (handle->handle, buffer, num_bytes,
 			bytes_written);
+
+	handle->space_left -= *bytes_written;
+	return result;
 }
 
 static GnomeVFSResult
